@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2014-2021 S[&]T, The Netherlands.
+# Copyright (C) 2014-2022 S[&]T, The Netherlands.
 #
 
 from __future__ import absolute_import, division, print_function
@@ -94,6 +94,15 @@ def _load_extension(name):
     return sys.modules[name]
 
 
+def _inspect_nargs(func):
+    try:
+        getargspec = inspect.getfullargspec
+    except AttributeError:
+        getargspec = inspect.getargspec
+
+    return len(getargspec(func).args)
+
+
 def create(configuration):
     options = config.parse(configuration.get("archive", {}), _ArchiveConfig)
     _ArchiveConfig.validate(options)
@@ -459,6 +468,110 @@ class Archive(object):
             properties.core = Struct()
         properties.core.metadata_date = self._database.server_time_utc()
 
+    def attach(self, paths, product_type=None, use_symlinks=None, verify_hash=False, use_current_path=False):
+        """Add a product to the archive using an existing metadata record in the database.
+
+        This function acts as the inverse of a strip(). A metadata record for this product should already exist in
+        the database and no product should exist for it in the archive.
+
+        The existing metadata record is found by performing a search based on product_type and physical_name.
+
+        Keyword arguments:
+        product_type     -- Product type of the product to ingest. If left unspecified, an attempt will be made to
+                            determine the product type automatically. By default, the product type will be determined
+                            automatically.
+        use_symlinks     -- If set to True, symbolic links to the original product will be stored in the archive
+                            instead of a copy of the original product. If set to None, the value of the corresponding
+                            archive wide configuration option will be used. By default, the archive configuration will
+                            be used.
+                            This option is ignored if use_current_path=True.
+        verify_hash      -- If set to True then, after the ingestion, the product in the archive will be matched against
+                            the hash from the metadata (only if the metadata contained a hash).
+        use_current_path -- Ingest the product by keeping the file(s) at the current path (which must be inside the
+                            root directory of the archive).
+                            This option is ignored if ingest_product=False.
+
+        """
+        if isinstance(paths, basestring):
+            paths = [paths]
+
+        if not paths:
+            raise muninn.Error("nothing to ingest")
+
+        # Use absolute paths to make error messages more useful, and to avoid broken links when ingesting a product
+        # using symbolic links.
+        paths = [os.path.realpath(path) for path in paths]
+
+        # Ensure that the set of files and / or directories that make up the product does not contain duplicate
+        # basenames.
+        basenames = [os.path.basename(path) for path in paths]
+        if len(set(basenames)) < len(basenames):
+            raise Error("basename of each part should be unique for multi-part products")
+
+        # Get the product type plug-in.
+        if product_type is None:
+            product_type = self.identify(paths)
+        plugin = self.product_type_plugin(product_type)
+
+        metadata = plugin.analyze(paths)
+        if hasattr(plugin, 'namespaces'):
+            namespaces = plugin.namespaces
+        else:
+            namespaces = []
+        for namespace in metadata:
+            if namespace != 'core' and namespace not in namespaces:
+                warnings.warn("plugin.namespaces does not contain '%s'" % namespace, DeprecationWarning)
+        if isinstance(metadata, (tuple, list)):
+            properties, tags = metadata
+        else:
+            properties, tags = metadata, []
+
+        # Determine physical product name.
+        if plugin.use_enclosing_directory:
+            physical_name = plugin.enclosing_directory(properties)
+        elif len(paths) == 1:
+            physical_name = os.path.basename(paths[0])
+        else:
+            raise Error("cannot determine physical name for multi-part product")
+
+        # Find product in catalogue
+        products = self.search(where="product_type == @product_type and physical_name == @physical_name",
+                               parameters={"product_type": product_type, "physical_name": physical_name},
+                               namespaces=plugin.namespaces)
+        if len(products) == 0:
+            raise Error("product with physical_name '%s' not found" % physical_name)
+        product = products[0]
+
+        # Determine archive path
+        if 'archive_path' in product.core:
+            raise Error("product with physical_name '%s' is already in the archive" % physical_name)
+        if use_current_path:
+            product.core.archive_path = self._storage.current_archive_path(paths)
+        else:
+            product.core.archive_path = plugin.archive_path(product)
+
+        # set archive_path and deactivate while we pull it in
+        metadata = {'active': False, 'archive_path': product.core.archive_path}
+        self.update_properties(Struct({'core': metadata}), product.core.uuid)
+
+        # Store the product into the archive.
+        use_enclosing_directory = plugin.use_enclosing_directory
+        self._storage.put(paths, product, use_enclosing_directory, use_symlinks)
+
+        # Verify product hash after copy
+        if verify_hash:
+            if self.verify_hash("uuid == @uuid", {"uuid": product.core.uuid}):
+                raise Error("ingested product has incorrect hash")
+
+        # Activate product.
+        metadata = {
+            'active': True,
+            'archive_date': self._database.server_time_utc(),
+        }
+        self.update_properties(Struct({'core': metadata}), product.core.uuid)
+
+        return properties
+
     def auth_file(self):
         """Return the path of the authentication file to download from remote locations"""
         return self._auth_file
@@ -569,7 +682,7 @@ class Archive(object):
         if format is not None:
             if re.match("[a-zA-Z]\\w*$", format) is None:
                 raise Error("invalid export format '%s'" % format)
-            export_method_name = export_method_name + "_" + format
+            export_method_name += "_" + format
 
         result = []
         products = self.search(where=where, parameters=parameters, namespaces=self.namespaces())
@@ -584,17 +697,27 @@ class Archive(object):
             #
             # Note the use of getattr() / hasattr() instead of a try + except AttributeError block, to avoid hiding
             # AttributeError instances raised by the plug-in.
-            plugin = self.product_type_plugin(product.core.product_type)
 
+            plugin = self.product_type_plugin(product.core.product_type)
             export_method = getattr(plugin, export_method_name, None)
+
             if export_method is not None:
-                exported_path = export_method(self, product, target_path)
+                if _inspect_nargs(export_method) == 5:
+                    def _export(paths):
+                        exported_path = export_method(self, product, target_path, paths)
+                        result.append(exported_path)
+                    self._storage.run_for_product(product, _export, plugin.use_enclosing_directory)
+                else:
+                    exported_path = export_method(self, product, target_path)
+                    result.append(exported_path)
+
             elif format is not None:
                 raise Error("export format '%s' not supported for product '%s' (%s)" %
                             (format, product.core.product_name, product.core.uuid))
+
             else:
                 exported_path = self._retrieve(product, target_path, False)
-            result.append(exported_path)
+                result.append(exported_path)
 
         return result
 
@@ -800,22 +923,20 @@ class Archive(object):
             if ingest_product:
                 use_enclosing_directory = plugin.use_enclosing_directory
                 self._storage.put(paths, properties, use_enclosing_directory, use_symlinks)
-
-                # Verify product hash after copy
-                if verify_hash:
-                    if self.verify_hash("uuid == @uuid", {"uuid": properties.core.uuid}):
-                        raise Error("ingested product has incorrect hash")
-
                 properties.core.archive_date = self._database.server_time_utc()
-        except Exception:
-            # Try to remove the entry for this product from the product catalogue.
-            self._database.delete_product_properties(properties.core.uuid)
-            raise
 
-        # Activate product.
-        properties.core.active = True
+        except Exception as e:
+            if not (isinstance(e, StorageError) and e.anything_stored):
+                # Try to remove the entry for this product from the product catalogue.
+                self._database.delete_product_properties(properties.core.uuid)
+
+            if isinstance(e, StorageError):
+                raise e.orig
+            else:
+                raise
+
+        # Update archive date.
         metadata = {
-            'active': properties.core.active,
             'archive_date': properties.core.archive_date,
         }
         self.update_properties(Struct({'core': metadata}), properties.core.uuid)
@@ -823,12 +944,24 @@ class Archive(object):
         # Set product tags.
         self._database.tag(properties.core.uuid, tags)
 
+        # Verify product hash after copy
+        if ingest_product and verify_hash:
+            if self.verify_hash("uuid == @uuid", {"uuid": properties.core.uuid}):
+                raise Error("ingested product has incorrect hash")
+
         # Run post create/ingest hooks (if defined by the product type plug-in or hook extensions).
         if not ingest_product:
             self._run_hooks('post_create_hook', plugin, properties)
         else:
             self._run_hooks('post_ingest_hook', plugin, properties, paths=paths)
 
+        # Activate product.
+        properties.core.active = True
+        metadata = {
+            'active': properties.core.active,
+        }
+        self.update_properties(Struct({'core': metadata}), properties.core.uuid)
+
         return properties
 
     def _hook_exists(self, hook_name, plugin):
@@ -846,12 +979,7 @@ class Archive(object):
         for plugin in plugins:
             hook_method = getattr(plugin, hook_name, None)
             if hook_method is not None:
-                try:
-                    getargspec = inspect.getfullargspec
-                except AttributeError:
-                    getargspec = inspect.getargspec
-
-                if len(getargspec(hook_method).args) == 4:
+                if _inspect_nargs(hook_method) == 4:
                     hook_method(self, properties, paths)
                 else:
                     hook_method(self, properties)
@@ -953,10 +1081,10 @@ class Archive(object):
             self.update_properties(Struct({'core': metadata}), product.core.uuid)
 
             def _pull(paths):
-                # reactivate and update size
+                # update archive_date, size
                 product_path = self._product_path(product)
                 size = self._storage.size(product_path)
-                metadata = {'active': True, 'archive_date': self._database.server_time_utc(), 'size': size}
+                metadata = {'archive_date': self._database.server_time_utc(), 'size': size}
                 self.update_properties(Struct({'core': metadata}), product.core.uuid)
 
                 # verify product hash.
@@ -973,11 +1101,20 @@ class Archive(object):
                 retrieve_files = remote.retrieve_function(self, product, verify_hash_download)
                 self._storage.put(None, product, use_enclosing_directory, use_symlinks=False,
                                   retrieve_files=retrieve_files, run_for_product=_pull)
-            except Exception:
-                # reset active/archive_path values
-                metadata = {'active': True, 'archive_path': None}
-                self.update_properties(Struct({'core': metadata}), product.core.uuid)
-                raise
+            except Exception as e:
+                if not (isinstance(e, StorageError) and e.anything_stored):
+                    # reset archive_path
+                    metadata = {'archive_path': None}
+                    self.update_properties(Struct({'core': metadata}), product.core.uuid)
+
+                if isinstance(e, StorageError):
+                    raise e.orig
+                else:
+                    raise
+
+            # activate product
+            metadata = {'active': True}
+            self.update_properties(Struct({'core': metadata}), product.core.uuid)
 
         return len(queue)
 
