@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2014-2021 S[&]T, The Netherlands.
+# Copyright (C) 2014-2022 S[&]T, The Netherlands.
 #
 
 from __future__ import absolute_import, division, print_function
@@ -470,8 +470,11 @@ class SQLiteBackend(object):
         except KeyError:
             raise Error("undefined namespace: \"%s\"" % namespace)
 
-    def _placeholder(self, name=""):
-        return "?" if not name else ":%s" % name
+    def _placeholder(self, name=None, arg=None):
+        if name is not None:
+            return ":%s" % name
+        else:
+            return "?"
 
     def _rewriter_property(self, column_name, subscript):
         # timestamp
