@@ -95,7 +95,9 @@ class FilesystemStorageBackend(StorageBackend):
         # strip archive root
         return os.path.relpath(abs_archive_path, start=os.path.realpath(self._root))
 
-    def put(self, paths, properties, use_enclosing_directory, use_symlinks=None, retrieve_files=None):
+    def put(self, paths, properties, use_enclosing_directory, use_symlinks=None,
+            retrieve_files=None, run_for_product=None):
+
         if use_symlinks is None:
             use_symlinks = self._use_symlinks
 
@@ -167,6 +169,10 @@ class FilesystemStorageBackend(StorageBackend):
                         tmp_product_path = os.path.join(tmp_path, physical_name)
                         os.rename(tmp_product_path, abs_product_path)
 
+                    # Run optional function on result
+                    if run_for_product is not None:
+                        self.run_for_product(properties, run_for_product, use_enclosing_directory)
+
             except EnvironmentError as _error:
                 raise Error("unable to transfer product to destination path '%s' [%s]" %
                             (abs_product_path, _error))
@@ -216,10 +222,10 @@ class FilesystemStorageBackend(StorageBackend):
             raise Error("unable to remove product '%s' (%s) [%s]" % (properties.core.product_name, properties.core.uuid,
                                                                      _error))
 
-    def move(self, product, archive_path):
+    def move(self, product, archive_path, paths=None):
         # Ignore if product already there
         if product.core.archive_path == archive_path:
-            return
+            return paths
 
         # Make target archive path
         abs_archive_path = os.path.realpath(os.path.join(self._root, archive_path))
@@ -228,3 +234,10 @@ class FilesystemStorageBackend(StorageBackend):
         # Move files there
         product_path = self.product_path(product)
         os.rename(product_path, os.path.join(abs_archive_path, product.core.physical_name))
+
+        # Optionally rewrite (local) paths
+        if paths is not None:
+            paths = [os.path.join(self._root, archive_path,
+                                  os.path.relpath(path, os.path.join(self._root, product.core.archive_path)))
+                     for path in paths]
+        return paths
