@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2014-2021 S[&]T, The Netherlands.
+# Copyright (C) 2014-2022 S[&]T, The Netherlands.
 #
 
 from __future__ import absolute_import, division, print_function
@@ -66,7 +66,7 @@ class TabulateWriter(PlainWriter):
         self._data.append(values)
 
     def footer(self):
-        print(tabulate.tabulate(self._data, headers=self._header, tablefmt=self._format))
+        print(tabulate.tabulate(self._data, headers=self._header, tablefmt=self._format, disable_numparse=True))
 
 
 class CSVWriter(PlainWriter):
