@@ -4,6 +4,7 @@ try:
 except ImportError:
     from ConfigParser import ConfigParser
 
+import datetime
 import glob
 import logging
 import os
@@ -42,12 +43,13 @@ CFG.read(u'test.cfg')
 STORAGE_BACKENDS = [s.strip() for s in CFG.get('DEFAULT', 'storage').split(',')]
 DATABASE_BACKENDS = [s.strip() for s in CFG.get('DEFAULT', 'database').split(',')]
 ARCHIVE_PATHS = [s.strip() for s in CFG.get('DEFAULT', 'archive_path').split(',')]
-USE_ENCLOSING_DIR = [s.strip()=='true' for s in CFG.get('DEFAULT', 'use_enclosing_dir').split(',')]
+USE_ENCLOSING_DIR = [s.strip() == 'true' for s in CFG.get('DEFAULT', 'use_enclosing_dir').split(',')]
 
 
 class MyNamespace(Mapping):
     hello = optional(Text)
 
+
 class MyNamespace2(Mapping):
     counter = optional(Integer)
 
@@ -135,6 +137,11 @@ STORAGE_CHECKERS = {
 }
 
 
+def _makedirs(path):
+    if not os.path.exists(path):
+        os.makedirs(path)
+
+
 # TODO merge fixtures into one with multiple parameters?
 
 @pytest.fixture(params=DATABASE_BACKENDS)
@@ -195,8 +202,8 @@ def archive(database, storage, use_enclosing_directory, archive_path):
     os.system('rm *.pyc -f')
 
     # create empty dirs (can't commit in git)
-    os.makedirs('data/multi/emptydir', exist_ok=True)
-    os.makedirs('data/multi/dir/emptydir', exist_ok=True)
+    _makedirs('data/multi/emptydir')
+    _makedirs('data/multi/dir/emptydir')
 
     # create clean archive
     with muninn.open('my_arch') as archive:
@@ -218,18 +225,18 @@ def archive(database, storage, use_enclosing_directory, archive_path):
 
 
 class TestArchive:
-    def _ingest_file(self, archive, use_symlinks=False, intra=False):
-        name = 'pi.txt'
+    def _ingest_file(self, archive, use_symlinks=False, intra=False, name=None):
+        name = name or 'pi.txt'
         path = 'data/%s' % name
         size = os.path.getsize(path)
 
-        if intra: # relative symlink within archive
+        if intra:  # relative symlink within archive
             dirpath = os.path.join(
                 archive._checker.root,
                 'one/two'
             )
 
-            os.makedirs(dirpath, exist_ok=True)
+            _makedirs(dirpath)
             shutil.copy(path, dirpath)
             path = os.path.join(dirpath, name)
 
@@ -238,10 +245,10 @@ class TestArchive:
             verify_hash=True,
             use_symlinks=use_symlinks
         )
-        path = os.path.join(archive._params['archive_path'], 'pi.txt')
+        path = os.path.join(archive._params['archive_path'], name)
 
         if archive._params['use_enclosing_directory']:
-            path = os.path.join(path, 'pi.txt')
+            path = os.path.join(path, name)
 
         assert archive._checker.exists(path, size)
 
@@ -252,8 +259,8 @@ class TestArchive:
 
             if intra:
                 # TODO remove this, as os.path.realpath already resolves
-                target_path = 'one/two/pi.txt'
-                dotdots = 0 # don't use relpath on purpose for comparison
+                target_path = 'one/two/' + name
+                dotdots = 0  # don't use relpath on purpose for comparison
                 if archive._params['use_enclosing_directory']:
                     dotdots += 1
                 if archive._params['archive_path']:
@@ -262,7 +269,7 @@ class TestArchive:
                     target_path = os.path.join('..', target_path)
                 assert os.readlink(source_path) == target_path
             else:
-                target_path = os.path.join(os.getcwd(), 'data/pi.txt')
+                target_path = os.path.join(os.getcwd(), 'data/' + name)
                 assert os.readlink(source_path) == target_path
 
         return properties
@@ -282,13 +289,13 @@ class TestArchive:
             assert 'cannot determine physical name for multi-part product' in str(excinfo)
             return
 
-        if intra: # relative symlinks within archive
+        if intra:  # relative symlinks within archive
             dirpath = os.path.join(
                 archive._checker.root,
                 'three/multi'
             )
 
-            os.makedirs(dirpath, exist_ok=True)
+            _makedirs(dirpath)
             for path in paths:
                 if os.path.isdir(path):
                     shutil.copytree(path, os.path.join(dirpath, os.path.basename(path)))
@@ -319,7 +326,7 @@ class TestArchive:
                 if intra:
                     # TODO remove this, as os.path.realpath already resolves
                     target_path = os.path.join('three/multi', os.path.basename(path))
-                    dotdots = 1 # enclosing
+                    dotdots = 1  # enclosing
                     if archive._params['archive_path']:
                         dotdots += 2
                     for i in range(dotdots):
@@ -389,7 +396,6 @@ class TestArchive:
     def test_ingest_file(self, archive):
         # copy
         self._ingest_file(archive)
-
         archive.remove()
 
         # symlink
@@ -399,12 +405,20 @@ class TestArchive:
             with pytest.raises(muninn.exceptions.Error) as excinfo:
                 self._ingest_file(archive, use_symlinks=True)
             assert 'storage backend does not support symlinks' in str(excinfo)
-
         archive.remove()
 
         # intra-archive symlink
         if archive._params['storage'] == 'fs':
             self._ingest_file(archive, use_symlinks=True, intra=True)
+        archive.remove()
+
+        # post hook/hash verification failure: check that active=False
+        with pytest.raises(ZeroDivisionError):  # hook raises exception for pi2.txt
+            self._ingest_file(archive, name='pi2.txt')
+        s = archive.search()
+        assert len(s) == 1
+        assert s[0].core.product_name == 'pi2.txt'
+        assert s[0].core.active is False
 
     def test_reingest(self, archive):
         if archive._params['storage'] == 'fs':
@@ -489,7 +503,7 @@ class TestArchive:
             assert not archive._checker.exists(path)
 
     def test_pull(self, archive):
-        self._pull(archive)
+        properties = self._pull(archive)
 
         path = os.path.join(archive._params['archive_path'], 'README')
         if archive._params['use_enclosing_directory']:
@@ -497,6 +511,22 @@ class TestArchive:
 
         assert archive._checker.exists(path)
 
+        # failing hook should result in inactive product
+        archive.strip("")
+        archive.update_properties(muninn.Struct({'mynamespace2': {'counter': 27}}), properties.core.uuid)
+
+        with pytest.raises(ZeroDivisionError):  # hook raises exception for counte==27
+            archive.pull("", verify_hash=True, verify_hash_download=True)
+        s = archive.search()
+        assert len(s) == 1
+        assert s[0].core.product_name == 'README'
+        assert s[0].core.active is False
+
+    def test_attach(self, archive):
+        archive.ingest(['data/pi.txt'])
+        archive.strip()
+        archive.attach(['data/pi.txt'])
+
     def test_search(self, archive):  # TODO move to TestQuery?
         properties = self._ingest_file(archive)
         uuid = properties.core.uuid
@@ -519,6 +549,12 @@ class TestArchive:
         s = archive.search('%s' % uuid)
         assert len(s) == 1
 
+        # 'covers' search on datetime field
+        s = archive.search('covers(@start, @stop, core.archive_date, core.archive_date)',
+                           parameters={'start': datetime.datetime.now() - datetime.timedelta(hours=24),
+                                       'stop': datetime.datetime.now()})
+        assert len(s) == 1
+
         # use uuid as boolean
         s = archive.search('%s and product_name == "pi.txt"' % uuid)
         assert len(s) == 1
@@ -584,7 +620,7 @@ class TestArchive:
             assert 'storage backend does not support symlinks' in str(excinfo)
 
     def test_retrieve_multi_file(self, archive):
-        if archive._params['use_enclosing_directory'] == True:
+        if archive._params['use_enclosing_directory'] is True:
             self._ingest_multi_file(archive)
 
             # copy
@@ -667,6 +703,24 @@ class TestArchive:
                 assert tf.getmember('multi/1.txt').size == 209
                 assert tf.getmember('multi/2.txt').size == 229
 
+    def test_export_default(self, archive):
+        if archive._params['use_enclosing_directory']:
+            self._ingest_file(archive)
+
+            with muninn.util.TemporaryDirectory() as tmp_path:
+                # no format passed, and plugin has no 'export' method: use default (retrieve)!
+                archive.export(target_path=tmp_path)
+
+                os.system('tree '+tmp_path)
+
+                path = os.path.join(
+                           tmp_path,
+                           'pi.txt'  # TODO does not add archive path?
+                       )
+
+                assert os.path.exists(path)
+                assert os.path.getsize(path) == 1015
+
     def test_rebuild_properties_file(self, archive):
         properties = self._ingest_file(archive)
 
@@ -781,18 +835,18 @@ class TestArchive:
     def test_summary(self, archive):
         product1 = archive.ingest(['data/a.txt'])
         year = product1.core.archive_date.year
-        time.sleep(1) # different dates
+        time.sleep(1)  # different dates
         product2 = archive.ingest(['data/b.txt'])
 
         # default summary (count all)
         data, headers = archive.summary()
         assert headers == ['count']
-        assert data == [(2,)] or data == [[2]] # TODO pg8000 gives list per row??
+        assert data == [(2,)] or data == [[2]]  # TODO pg8000 gives list per row??
 
         # aggregate size.sum
-        data, headers = archive.summary(aggregates=['core.size.sum']) # TODO doesn't work without core prefix
+        data, headers = archive.summary(aggregates=['core.size.sum'])  # TODO doesn't work without core prefix
         assert headers == ['count', 'core.size.sum']
-        assert data == [(2, 2030)] or data == [[2, 2030]] # TODO pg8000
+        assert data == [(2, 2030)] or data == [[2, 2030]]  # TODO pg8000
 
         # non-core namespace
         data, headers = archive.summary(aggregates=['mynamespace2.counter.avg'])
@@ -833,7 +887,7 @@ class TestQuery:
         archive.update_properties(muninn.Struct({'mynamespace': {'hello': 'hohoho'}}), self.uuid_a, True)
         archive.update_properties(muninn.Struct({'mynamespace': {'hello': 'hohoho'}}), self.uuid_b, True)
 
-        polygon = Polygon([LinearRing([Point(0,0), Point(4,0), Point(4,4), Point(0,4)])])
+        polygon = Polygon([LinearRing([Point(0, 0), Point(4, 0), Point(4, 4), Point(0, 4)])])
         archive.update_properties(muninn.Struct({'core': {'footprint': polygon}}), self.uuid_c, True)
 
         archive.link(self.uuid_b, [self.uuid_a])
@@ -869,9 +923,11 @@ class TestQuery:
         s = archive.search('not is_derived_from(is_derived_from(physical_name==\"a.txt\"))')
         assert len(s) == 2
 
-        s = archive.search('is_derived_from(physical_name==\"a.txt\") or is_derived_from(is_derived_from(physical_name==\"a.txt\"))')
+        s = archive.search('is_derived_from(physical_name==\"a.txt\") or ' +
+                           'is_derived_from(is_derived_from(physical_name==\"a.txt\"))')
         assert len(s) == 2
-        s = archive.search('not (is_derived_from(physical_name==\"a.txt\") or is_derived_from(is_derived_from(physical_name==\"a.txt\")))')
+        s = archive.search('not (is_derived_from(physical_name==\"a.txt\") or ' +
+                           'is_derived_from(is_derived_from(physical_name==\"a.txt\")))')
         assert len(s) == 1
 
     def test_IsSourceOf(self, archive):
@@ -902,14 +958,14 @@ class TestQuery:
         assert len(s) == 0
         s = archive.search('\"hiya\"==mynamespace.hello')
         assert len(s) == 0
-        s = archive.search('not mynamespace.hello==\"hiya\"') # TODO move to logic operator tests
+        s = archive.search('not mynamespace.hello==\"hiya\"')  # TODO move to logic operator tests
         assert len(s) == 3
         s = archive.search('mynamespace.hello!=\"hiya\"')
         assert len(s) == 3
-        s = archive.search('not mynamespace.hello!=\"hiya\"') # TODO move to logic operator tests, etc.
+        s = archive.search('not mynamespace.hello!=\"hiya\"')  # TODO move to logic operator tests, etc.
         assert len(s) == 0
 
-        s = archive.search('size!=0') # TODO to better place
+        s = archive.search('size!=0')  # TODO to better place
         assert len(s) == 3
         s = archive.search('0<size')
         assert len(s) == 3
